using System;
using Bierhallen.DAL;
using System.Collections.Generic;


namespace Bierhallen.Domain
{
    public class Brouwer
    {
        #region "attributen"
        private int _brouwernr;
        private string _naam;
        private string _adres;
        private string _postcode;
        private Gemeente _gemeente;
        private int _omzet;
#endregion

        #region "constructors"

        public Brouwer() { }
        public Brouwer(string naam, string adres, string postcode, int omzet)
        {
            Naam = naam;
            Adres = adres;
            Postcode = postcode;
            Omzet = omzet;
        }

        public Brouwer(int brouwernr, string naam, string adres, string postcode, int omzet)
            : this(naam, adres, postcode, omzet)
        {
            Brouwernr = brouwernr;
        }
        #endregion

        #region "Properties"
        

        public int Brouwernr
        {
            get { return _brouwernr; }
            set { _brouwernr = value; }
        }
        public string Naam
        {
            get { return _naam; }
            set
            {
                if (String.IsNullOrEmpty(value))
                    throw new ArgumentException("Naam is verplicht");
                else if (value.Length > 50)
                    throw new ArgumentException("Naam te lang");
                else
                { _naam = value; }
            }
        }
        public string Adres
        {
            get { return _adres; }
            set { _adres = value; }
        }
        public string Postcode
        {
            get { return _postcode; }
            set { _postcode = value; }
        }

        public Gemeente Gemeente
        {
            get
            {
                //Lazy load
                if (_gemeente == null)
                    _gemeente = BrouwersDAL.Instance.GetGemeente(_postcode);
                return _gemeente;
            }
            private set
            {
                _gemeente = value;
            }
        }

        public int Omzet
        {
            get { return _omzet; }
            set { _omzet = value; }
        }
        public override string ToString()
        {
            return _naam;
        }
        #endregion
       
#region "Methodes"

        internal void Validate()
        {
            //Internal types or members are accessible only within files in the same assembly (namespace)
            //hier kan je validatie code schrijven. Moet aangeroepen wordn alvorens je dingen naar de database schrijft.
            //Je zou bvb een business rule kunnen toevoegen : als brouwer in Gent woont, dan maximale omzet 2000 euro
            if (_postcode == "9000" && _omzet > 2000)
                throw new Exception("Omzet te groot");
        }
#endregion


    }
}

