using System;
using System.Data;
using System.Collections.Generic;
using System.Web.Configuration;  //toegang tot web.config
using System.Data.SqlClient;
using Bierhallen.Domain;

namespace Bierhallen.DAL
{

    public class BrouwersDAL
    {
        private string _connectionString;
        //Singleton pattern
        static private BrouwersDAL _instance = null;
        static public BrouwersDAL Instance
        {
            get
            {
                if (_instance == null)
                    _instance = new BrouwersDAL();
                return _instance;
            }
        }
        private BrouwersDAL()
        {
            _connectionString = WebConfigurationManager.ConnectionStrings["Bieren"].ConnectionString;
        }

#region "methodes"
        
     
        public List<Brouwer> GetBrouwers()
        {
            using (SqlConnection oConn = new SqlConnection(_connectionString)) {
                string strSql = "Select * from brouwers order by naam";
                SqlCommand oCmd = new SqlCommand(strSql, oConn);
                oConn.Open();
                // execute query
                SqlDataReader oReader = oCmd.ExecuteReader();
                List<Brouwer> brouwers = null;
                brouwers = GetBrouwerCollectionFromReader(oReader);
                oReader.Close();
                return brouwers;
            }
        }         
        
        public Brouwer GetBrouwer(int brouwernr)
        {
            using (SqlConnection oConn = new SqlConnection(_connectionString)) {
                //string strSql = "Select * from brouwers where brouwernr = " + brouwernr.ToString();  //kan gehackt worden
                string strSql = "Select * from brouwers where brouwernr = @brouwernr";
                SqlCommand oCmd = new SqlCommand(strSql, oConn);
                oCmd.Parameters.Add(new SqlParameter("@brouwernr", brouwernr));
                oConn.Open();
                SqlDataReader oReader = oCmd.ExecuteReader(CommandBehavior.SingleRow);
                Brouwer brouwer = null;
                if (oReader.Read())
                    brouwer = GetBrouwerFromReader(oReader);
                oReader.Close();
                return brouwer;
            }
        }

        public bool DeleteBrouwer(int brouwernr)
        {
            using (SqlConnection oConn = new SqlConnection(_connectionString))
            {
                string sqlString = "delete from brouwers where brouwernr=@brouwernr";
                SqlCommand oCmd = new SqlCommand(sqlString, oConn);
                oCmd.Parameters.Add(new SqlParameter("@brouwernr", brouwernr));
                oConn.Open();
                int rowsaffected = oCmd.ExecuteNonQuery();
                return (rowsaffected == 1);
            }
        }

        public bool UpdateBrouwer(Brouwer brouwer)
        {
            using (SqlConnection oConn = new SqlConnection(_connectionString)) {
                string sqlString = "Update brouwers set naam=@naam, adres=@adres,postcode=@postcode, omzet=@omzet where brouwernr=@brouwernr";
                SqlCommand oCmd = new SqlCommand(sqlString, oConn);
                oCmd.Parameters.Add(new SqlParameter("@naam", brouwer.Naam));
                oCmd.Parameters.Add(new SqlParameter("@adres", brouwer.Adres));
                oCmd.Parameters.Add(new SqlParameter("@postcode", brouwer.Postcode));
                if (brouwer.Omzet != 0)
                    oCmd.Parameters.Add(new SqlParameter("@omzet", brouwer.Omzet));
                else
                    oCmd.Parameters.Add(new SqlParameter("@omzet", DBNull.Value));
                oCmd.Parameters.Add(new SqlParameter("@brouwernr", brouwer.Brouwernr));
                oConn.Open();
                int rowsAffected = oCmd.ExecuteNonQuery();
                return (rowsAffected==1);
            }            
        }

        public int InsertBrouwer(Brouwer brouwer)
        {
            using (SqlConnection oConn = new SqlConnection(_connectionString)){
                string sqlString = "Insert into brouwers (naam, adres, postcode,omzet) "
                    +"values(@naam,@adres,@postcode,@omzet); select SCOPE_IDENTITY()"; // select last id
                SqlCommand oCmd = new SqlCommand(sqlString, oConn);
                oCmd.Parameters.Add(new SqlParameter("@naam", SqlDbType.NVarChar, 50));
                oCmd.Parameters["@naam"].Value = brouwer.Naam;
                oCmd.Parameters.Add(new SqlParameter("@adres", SqlDbType.NVarChar, 50)).Value = brouwer.Adres;
                oCmd.Parameters.Add(new SqlParameter("@postcode", SqlDbType.SmallInt));
                oCmd.Parameters["@postcode"].Value = Int16.Parse(brouwer.Postcode);
                if (brouwer.Omzet != 0)
                    oCmd.Parameters.Add(new SqlParameter("@omzet", brouwer.Omzet));
                else
                    oCmd.Parameters.Add(new SqlParameter("@omzet", DBNull.Value));
                 oConn.Open();
                 int brouwernr = (int)(decimal)oCmd.ExecuteScalar();
                 return brouwernr;
            }
        }
          

        public int CountBrouwers()
        {
            using (SqlConnection oConn = new SqlConnection(_connectionString))
            {
                string sqlString = "select count(*) from brouwers";
                SqlCommand oCmd = new SqlCommand(sqlString, oConn);
                oConn.Open();
                return (int)oCmd.ExecuteScalar();
            }
        }

        public Gemeente GetGemeente(string postcode)
        {
            using (SqlConnection oConn = new SqlConnection(_connectionString))
            {
                string sqlString = "Select gemeente from gemeenten where postcode = @postcode";
                SqlCommand oCmd = new SqlCommand(sqlString, oConn);
                oCmd.Parameters.Add(new SqlParameter("@postcode", postcode));
                oConn.Open();
                //Create a collection for all the brouwers records.
                SqlDataReader oReader = oCmd.ExecuteReader(CommandBehavior.SingleRow);
                Gemeente gemeente = GetGemeenteFromReader(oReader);
                oReader.Close();
                return gemeente;
            }
        }

        public List<Gemeente> GetGemeenten()
        {
            using (SqlConnection oConn = new SqlConnection(_connectionString))
            {
                string sqlString = "Select * from gemeenten order by gemeente";
                SqlCommand oCmd = new SqlCommand(sqlString, oConn);
                oConn.Open();
                SqlDataReader oReader = oCmd.ExecuteReader();
                List<Gemeente> gemeenten = GetGemeenteCollectionFromReader(oReader);
                oReader.Close();
                return gemeenten;
            }

        } 
#endregion

#region "OR mappers tabel Brouwers"
        protected Brouwer GetBrouwerFromReader(IDataRecord oReader)
        {
            Brouwer brouwer = new Brouwer();
            brouwer.Brouwernr = (int)oReader["brouwernr"];
            brouwer.Naam = oReader.GetString(1);
            brouwer.Adres = (string)oReader["adres"];
            brouwer.Postcode = (string)oReader["postcode"];
            if( oReader["omzet"] != DBNull.Value)
                brouwer.Omzet = (int)oReader["omzet"];
            return brouwer;
        }

        protected  List<Brouwer> GetBrouwerCollectionFromReader(IDataReader oReader)
        {
            List<Brouwer> brouwers = new List<Brouwer>();
            while( oReader.Read() )
                // read line by line and map to an object
                brouwers.Add(GetBrouwerFromReader(oReader));
          
            return brouwers;
        }

        protected Gemeente GetGemeenteFromReader(IDataRecord oReader)
        {
            
            return new Gemeente((string)oReader["postcode"], (string)oReader["gemeente"]);
        }

        protected List<Gemeente> GetGemeenteCollectionFromReader(IDataReader oReader)
        {
            List<Gemeente> gemeenten = new List<Gemeente>();
            while (oReader.Read())
            {
                gemeenten.Add(GetGemeenteFromReader(oReader));
            }
            return gemeenten;
        }
#endregion
    }

}
